@@ -11,12 +11,10 @@ partlyObservedNetwork <-
   ## PRIVATE MEMBERS
   ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   private = list(
-    Y        = NULL, # adjacency matrix
+    Y        = NULL, # adjacency matrix (sparse encoding: NA and 0 are not encoded)
     X        = NULL, # the covariates matrix
     phi      = NULL, # the covariates array
     directed = NULL, # directed network of not
-    obs      = NULL, # collection of observed dyads
-    miss     = NULL, # collection of missing dyads
     R        = NULL, # the sampling matrix (sparse encoding)
     S        = NULL  # the (anti) sampling matrix (sparse encoding)
   ),
@@ -40,17 +38,18 @@ partlyObservedNetwork <-
     covarArray = function(value) {private$phi},
     #' @field covarMatrix the matrix of covariates
     covarMatrix = function(value) {if (missing(value)) return(private$X) else  private$X <- value},
-    #' @field missingDyads array indices of missing dyads
-    missingDyads    = function(value) {private$miss},
-    #' @field observedDyads array indices of observed dyads
-    observedDyads   = function(value) {private$obs},
     #' @field samplingMatrix matrix of observed and non-observed edges
-    samplingMatrix  = function(value) {private$R},
+    samplingMatrix = function(value) {private$R},
     #' @field samplingMatrixBar matrix of observed and non-observed edges
     samplingMatrixBar  = function(value) {private$S},
     #' @field observedNodes a vector of observed and non-observed nodes (observed means at least one non NA value)
     observedNodes   = function(value) {
-      (rowSums(private$S) + colSums(private$S)) == 0
+      if (private$directed) {
+        res <- rowSums(private$R) == (self$nbNodes - 1)
+      } else {
+        res <- (rowSums(private$R | t(private$R))) == (self$nbNodes - 1)
+      }
+      res
      }
   ),
   ## %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -81,21 +80,22 @@ partlyObservedNetwork <-
 
       ## sets of observed / unobserved dyads
       if (private$directed) {
-        dyads <- upper.tri(adjacencyMatrix) | lower.tri(adjacencyMatrix)
+        dyads <- .row(dim(adjacencyMatrix)) != .col(dim(adjacencyMatrix))
       } else {
-        dyads <- upper.tri(adjacencyMatrix)
+        dyads <- .row(dim(adjacencyMatrix)) < .col(dim(adjacencyMatrix))
       }
       ## where are my observations?
-      private$obs   <- which(!is.na(adjacencyMatrix) & dyads, arr.ind = TRUE )
-      private$miss  <- which( is.na(adjacencyMatrix) & dyads, arr.ind = TRUE )
+      NAs   <- is.na(adjacencyMatrix)
+      obs   <- which(!NAs & dyads, arr.ind = TRUE )
+      miss  <- which( NAs & dyads, arr.ind = TRUE )
       ## where are my non-zero entries?
-      nzero <- which(!is.na(adjacencyMatrix) & adjacencyMatrix != 0 & dyads, arr.ind = TRUE)
+      nzero <- which(!NAs & adjacencyMatrix != 0 & dyads, arr.ind = TRUE)
 
       ## sampling matrix (indicating who is observed)
-      private$R <- Matrix::sparseMatrix(private$obs[,1] , private$obs[,2] ,x = 1, dims = dim(adjacencyMatrix))
-      private$S <- Matrix::sparseMatrix(private$miss[,1], private$miss[,2],x = 1, dims = dim(adjacencyMatrix))
+      private$R <- Matrix::sparseMatrix(obs[,1] , obs[,2] ,x = 1, dims = dim(adjacencyMatrix))
+      private$S <- Matrix::sparseMatrix(miss[,1], miss[,2],x = 1, dims = dim(adjacencyMatrix))
       ## network matrix (only none zero, non NA values)
-      private$Y   <- Matrix::sparseMatrix(nzero[,1], nzero[,2], x = 1, dims = dim(adjacencyMatrix))
+      private$Y <- Matrix::sparseMatrix(nzero[,1], nzero[,2], x = 1, dims = dim(adjacencyMatrix))
 
     },
     #' @description method to cluster network data with missing value
@@ -116,7 +116,7 @@ partlyObservedNetwork <-
       connected   <- setdiff(1:n, unconnected)
       A <- A[connected,connected]
 
-      ## normalized absolute Laplacian with Gaussian kernel
+      ## normalized Laplacian
       D <- 1/sqrt(rowSums(abs(A)))
       L <- sweep(sweep(A, 1, D, "*"), 2, D, "*")
 ##      U <- base::svd(L, nu = max(vBlocks), nv = 0)$u
@@ -141,23 +141,25 @@ partlyObservedNetwork <-
     #' @description basic imputation from existing clustering
     #' @param type a character, the type of imputation. Either "median" or "average"
     imputation = function(type = c("median", "average", "zero")) {
-      adjacencyMatrix <- private$Y
+      adjMat <- private$Y
       if (!is.null(private$phi)) {
-        y <- as.vector(adjacencyMatrix[self$observedDyads])
-        X <- cbind(1, apply(private$phi, 3, function(x) x[self$observedDyads]))
+        obs <- which(private$R != 0)
+        y <- as.vector(adjMat[obs])
+        X <- cbind(1, apply(private$phi, 3, function(x) x[obs]))
 ### TODO: make it work for other model than Bernoulli / family than binomial
-        adjacencyMatrix[self$observedDyads] <- .logistic(residuals(glm.fit(X, y, family = binomial())))
+        adjMat[obs] <- .logistic(residuals(glm.fit(X, y, family = binomial())))
       }
-      suppressMessages(adjacencyMatrix[self$missingDyads] <-
+      miss <- which(private$R == 0)
+      suppressMessages(adjMat[miss] <-
         switch(match.arg(type),
-               "average"  = mean(adjacencyMatrix, na.rm = TRUE),
-               "median"   = median(adjacencyMatrix, na.rm = TRUE),
+               "average"  = mean(adjMat, na.rm = TRUE),
+               "median"   = median(adjMat, na.rm = TRUE),
                "zero"     = 0
         ))
       if (!private$directed)
-        suppressMessages(adjacencyMatrix[lower.tri(adjacencyMatrix)] <- Matrix::t(adjacencyMatrix)[lower.tri(adjacencyMatrix)])
+        suppressMessages(adjMat[lower.tri(adjMat)] <- Matrix::t(adjMat)[lower.tri(adjMat)])
 
-      adjacencyMatrix
+      adjMat
     }
   )
 )
